#include <dlog.h>

#include "view/main_window.h"
#include "view/sink_wgt.h"
#include "view/source_wgt.h"
#include "view/hover_window.h"
#include "$(appName).h"

#define HOVER_WND_REL_WIDTH		0.8
#define HOVER_WND_REL_HEIGHT	0.3
#define SINK_WGT_X_Y_OFFSET		0.08

const static char *source_wgt_text =
		"Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna " \
		"aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis" \
		" aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat" \
		" cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum";

static struct {
	Evas_Object *win;
	Evas_Object *win_bg;
	Evas_Object *source_label;
	Evas_Object *sink_entry;
	Evas_Object *hover;
} s_view_data = {
	.win = NULL,
	.win_bg = NULL,
	.source_label = NULL,
	.sink_entry = NULL,
	.hover = NULL
};


static void
main_window_delete_cb(void *data, Evas_Object *obj, void *event_info);

static void
__get_main_window_dimensions(Evas_Coord *w, Evas_Coord  *h);

static bool
__hit_test(Evas_Object *obj, int hit_x, int hit_y);

static void
__hw_back_button_cb(void *data, Evas_Object *obj, void *event_info);

bool
create_main_window(void)
{
	int screen_width = 0, screen_height = 0;
	s_view_data.win = elm_win_util_standard_add(PACKAGE, PACKAGE);

	if (!s_view_data.win) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failure allocating resource: main window");
		return false;
	}

	elm_win_conformant_set(s_view_data.win, EINA_FALSE);
	elm_win_autodel_set(s_view_data.win, EINA_TRUE);
	elm_win_screen_size_get(s_view_data.win, NULL, NULL, &screen_width, &screen_height);
	evas_object_resize(s_view_data.win, screen_width, screen_height);

	evas_object_smart_callback_add(s_view_data.win, "delete, request", main_window_delete_cb, NULL);

	evas_object_pass_events_set(s_view_data.win, EINA_FALSE);

	/* add some background to hold sink widget image: */
	s_view_data.win_bg = elm_bg_add(s_view_data.win);
	if (!s_view_data.win_bg) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failure allocating resource: main window background");
		return false;
	}

	/* Show window after base gui is set up */
	evas_object_show(s_view_data.win);

	eext_object_event_callback_add(s_view_data.win, EEXT_CALLBACK_BACK, __hw_back_button_cb, NULL);

	return true;
}

void
destroy_main_window(void)
{
	evas_object_del(s_view_data.win);
}

bool
create_main_window_source_wgt(const char  *theme_extension)
{
	Evas_Coord main_window_width = 0, main_window_height = 0;
	__get_main_window_dimensions(&main_window_width, &main_window_height);

	if (!(s_view_data.source_label = _source_wgt_create(s_view_data.win))) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failure allocating resource: source wgt");
		return false;
	}

	elm_object_text_set(s_view_data.source_label, source_wgt_text);

	evas_object_move(s_view_data.source_label, 0, 0);
	evas_object_resize(s_view_data.source_label, main_window_width, main_window_height/2);

	/* This is how we use 'cnp_custom 'theme extension from edje file: */
	elm_theme_extension_add(NULL, theme_extension);
	elm_object_style_set(s_view_data.source_label, "cnp_custom");

	evas_object_show(s_view_data.source_label);

	return true;
}

bool
create_main_window_sink_entry_wgt(const char  *bg_image)
{
	Evas_Coord main_window_width = 0, main_window_height = 0;
	__get_main_window_dimensions(&main_window_width, &main_window_height);

	evas_object_show(s_view_data.win_bg);

	if (!(s_view_data.sink_entry = _create_sink_wgt(s_view_data.win))) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failure allocating resource: sink wgt");
		return false;
	}

	/* now let's set up previously prepared background, add some image file: */
	evas_object_move(s_view_data.win_bg, 0, main_window_height/2);
	evas_object_resize(s_view_data.win_bg, main_window_width, main_window_height/2);
	elm_bg_file_set(s_view_data.win_bg, bg_image, NULL);
	elm_bg_option_set(s_view_data.win_bg, ELM_BG_OPTION_STRETCH);

	/* on top of that, let's move our entry widget: */
	evas_object_move(s_view_data.sink_entry,
						SINK_WGT_X_Y_OFFSET*main_window_width,
						(1.0 + SINK_WGT_X_Y_OFFSET)*main_window_height/2);

	evas_object_resize(s_view_data.sink_entry,
						(1.0 - 2.0*SINK_WGT_X_Y_OFFSET)*main_window_width,
						(1.0 - 2.0*SINK_WGT_X_Y_OFFSET)*main_window_height/2);

	evas_object_show(s_view_data.sink_entry);

	return true;
}

bool
create_main_window_hover(const char  *theme_extension)
{
	Evas_Coord main_window_width = 0, main_window_height = 0;
	__get_main_window_dimensions(&main_window_width, &main_window_height);
	dlog_print(DLOG_DEBUG, LOG_TAG, "W: %d H: %d", main_window_width, main_window_height);

	s_view_data.hover = _create_hover_window(s_view_data.win,
											HOVER_WND_REL_WIDTH*main_window_width,
											HOVER_WND_REL_HEIGHT*main_window_height,
											theme_extension);

	if (!s_view_data.hover) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Main window: hover creation error");
		return false;
	}

	_set_hover_window_coordinates(0, 0);

	return true;
}

bool
test_main_window_source_wgt_hit(int x, int y)
{
	return __hit_test(s_view_data.source_label, x, y);
}

bool
test_main_window_sink_wgt_hit(int x, int y)
{
	return __hit_test(s_view_data.sink_entry, x, y);
}

void
paste_main_window_hover_text(void)
{
	_paste_hover_window_text(_source_wgt_text_get(s_view_data.source_label));
}

void
paste_window_sink_wgt_text(void)
{
	_paste_sink_wgt_text(s_view_data.sink_entry);
	/* after text has been copied, clear selection in clipboard: */
	elm_object_cnp_selection_clear(s_view_data.source_label, ELM_SEL_TYPE_CLIPBOARD);
}

static void
main_window_delete_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

static void
__get_main_window_dimensions(Evas_Coord *w, Evas_Coord *h)
{
	evas_object_geometry_get(s_view_data.win, NULL, NULL, w, h);
}

static bool
__hit_test(Evas_Object *obj, int hit_x, int hit_y)
{
	Evas_Coord x = 0, y = 0, w = 0, h = 0;
	evas_object_geometry_get(obj, &x, &y, &w, &h);

	return hit_x >= x && hit_x < (x + w) && hit_y >= y && hit_y < (y + h);
}

static void
__hw_back_button_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_win_lower(s_view_data.win);
}
